#lang racket

;; ----------------------------------------------------------------------

(define values '((a . 11) (b . 24) (c . 3)))

(define lookup
  (lambda (var env)
    (if (assoc var env)
        (cdr (assoc var env))
        (error 'lookup "invalid variable reference ~a" var))))

(define get-value
  (lambda (match var)     ; have to pass var for error case
    (if match
        (cdr match)
        (error 'lookup "invalid variable reference ~a" var))))

;; Now we can implement lookup by calling get-value

;; (define lookup
;;   (lambda (var env)
;;     (get-value (assoc var env) var)))

;; We can even use program derivation to eliminate get-value!

;; (define lookup
;;   (lambda (var env)
;;     ((lambda (match)    ; no need to pass var now
;;        (if match
;;            (cdr match)
;;            (error 'lookup "invalid variable reference ~a" var)))
;;      (assoc var env))))

;; Keep this idea in mind for later...

;; ----------------------------------------------------------------------
;; for demonstration only

(let ((x 3)) 
  (+ x (* x 10)))

(let ((x 3)
      (y 5))
  (+ y (* x 10)))

(let () 1)

;; ----------------------------------------------------------------------

;; Implementing lookup with a let expression

;; (define lookup
;;   (lambda (var env)
;;     (let ((match (assoc var env)))
;;       (if match
;;           (cdr match)
;;           (error 'lookup "invalid variable reference ~a" var)))))

;; Racket translates this code into the final version of lookup
;; shown earlier!

;; ----------------------------------------------------------------------
;; What do you think the following code returns?
;; Uncomment this code and give it a try!

;; (define x 5)
;; 
;; (+ (let ((x 3)) 
;;      (+ x (* x 10)))
;;    x)

;; ----------------------------------------------------------------------
